@@ -1,7 +1,7 @@
 /*
-$Id: dilib.c 8746 2004-03-16 21:29:20Z kevin $
-$Source: /home/bll/DI/RCS/dilib.c,v $
-Copyright 1994-2003 Brad Lanam, Walnut Creek, CA
+$Id: dilib.c,v 1.30 2005-10-29 08:55:03-07 bll Exp $
+$Source: /export/home/bll/DI/RCS/dilib.c,v $
+Copyright 1994-2005 Brad Lanam, Walnut Creek, CA
 */
 
 /********************************************************/
@@ -209,6 +209,7 @@ static char *chkMountOptions _((char *, char *));
         _statfs_4arg) && \
         ! defined (_lib_getmntinfo) && \
         ! defined (_lib_getfsstat) && \
+        ! defined (_lib_getvfsstat) && \
         ! defined (_lib_mntctl) && \
         ! defined (_lib_getmnt)
 # if defined (MOUNTED)
@@ -244,6 +245,7 @@ static char *chkMountOptions _((char *, char *));
     ! defined (_lib_statvfs) && \
     ! defined (_lib_getmntinfo) && \
     ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
     ! defined (_lib_getmnt) /* SYSV.3 */
 # if ! defined (UBSIZE)
 #  if defined (BSIZE)
@@ -510,7 +512,9 @@ chkMountOptions (mntopts, str)
 
 #if _lib_statvfs && \
     ! defined (_lib_getmntinfo) && \
-    ! defined (_lib_getfsstat)
+    ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
+    ! defined (_lib_GetVolumeInformation)
 
 /*
  * di_getDiskInfo
@@ -595,6 +599,7 @@ di_getDiskInfo (diskInfo, diCount)
     ! defined (_lib_mntctl) && \
     ! defined (_lib_getmntinfo) && \
     ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
     ! defined (_lib_getmnt) && \
     ! defined (_lib_GetDiskFreeSpace) && \
     ! defined (_lib_GetDiskFreeSpaceEx) && \
@@ -675,6 +680,7 @@ di_getDiskEntries (diskInfo, diCount)
     ! defined (_lib_statvfs) && \
     ! defined (_lib_getmntinfo) && \
     ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat) && \
     ! defined (_lib_getmnt)
 
 /*
@@ -758,7 +764,6 @@ di_getDiskInfo (diskInfo, diCount)
 #endif /* _statfs_4arg */
 
 
-/* prefer windows API over cygwin */
 #if _lib_getmntent && \
     _lib_setmntent && \
     _lib_endmntent && \
@@ -868,7 +873,8 @@ di_getDiskEntries (diskInfo, diCount)
 #endif /* _lib_getmntent && _lib_setmntent && _lib_endmntent */
 
 #if _lib_getmntinfo && \
-    ! defined (_lib_getfsstat)
+    ! defined (_lib_getfsstat) && \
+    ! defined (_lib_getvfsstat)
 
 /*
  * di_getDiskEntries
@@ -1162,7 +1168,8 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif /* _lib_getmntinfo */
 
-#if _lib_getfsstat
+#if _lib_getfsstat && \
+    ! defined (_lib_getvfsstat)
 
 /*
  * di_getDiskEntries
@@ -1291,6 +1298,121 @@ di_getDiskEntries (diskInfo, diCount)
 
 #endif /* _lib_getfsstat */
 
+#if _lib_getvfsstat
+
+/*
+ * di_getDiskEntries
+ *
+ * NetBSD
+ *
+ */
+
+int
+# if _proto_stdc
+di_getDiskEntries (di_DiskInfo **diskInfo, int *diCount)
+# else
+di_getDiskEntries (diskInfo, diCount)
+    di_DiskInfo **diskInfo;
+    int *diCount;
+# endif
+{
+    di_DiskInfo     *diptr;
+    int             count;
+    int             idx;
+    long            bufsize;
+    struct statvfs  *mntbufp;
+    struct statvfs  *sp;
+
+    if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: getvfsstat\n"); }
+    count = getvfsstat ((struct statvfs *) NULL, 0, ST_NOWAIT);
+    if (count < 1)
+    {
+        fprintf (stderr, "Unable to do getvfsstat () errno %d\n", errno);
+        return -1;
+    }
+    bufsize = sizeof (struct statvfs) * count;
+    mntbufp = malloc (bufsize);
+    memset ((char *) mntbufp, '\0', sizeof (struct statvfs) * count);
+    count = getvfsstat (mntbufp, bufsize, ST_NOWAIT);
+
+    *diCount = count;
+    *diskInfo = (di_DiskInfo *) malloc (sizeof (di_DiskInfo) * count);
+    if (*diskInfo == (di_DiskInfo *) NULL)
+    {
+        fprintf (stderr, "malloc failed for diskInfo. errno %d\n", errno);
+        return -1;
+    }
+    memset ((char *) *diskInfo, '\0', sizeof (di_DiskInfo) * count);
+
+    for (idx = 0; idx < count; idx++)
+    {
+        diptr = *diskInfo + idx;
+        diptr->printFlag = DI_PRNT_OK;
+        diptr->isLocal = FALSE;
+        diptr->isReadOnly = FALSE;
+
+        sp = mntbufp + idx;
+
+# if defined (MNT_RDONLY)
+        if ((sp->f_flag & MNT_RDONLY) == MNT_RDONLY)
+        {
+            diptr->isReadOnly = TRUE;
+        }
+# endif
+# if defined (MNT_LOCAL)
+        if ((sp->f_flag & MNT_LOCAL) == MNT_LOCAL)
+        {
+            diptr->isLocal = TRUE;
+        }
+# endif
+        convertMountOptions (sp->f_flag, diptr);
+        trimChar (diptr->options, ',');
+
+        if (sp->f_frsize == 0 && sp->f_bsize != 0)
+        {
+            diptr->blockSize = (_fs_size_t) sp->f_bsize;
+        }
+        else
+        {
+            diptr->blockSize = (_fs_size_t) sp->f_frsize;
+        }
+        diptr->totalBlocks = (_fs_size_t) sp->f_blocks;
+        diptr->freeBlocks = (_fs_size_t) sp->f_bfree;
+        diptr->availBlocks = (_fs_size_t) sp->f_bavail;
+
+        diptr->totalInodes = sp->f_files;
+        diptr->freeInodes = sp->f_ffree;
+        diptr->availInodes = sp->f_ffree;
+
+        strncpy (diptr->special, sp->f_mntfromname, DI_SPEC_NAME_LEN);
+        strncpy (diptr->name, sp->f_mntonname, DI_NAME_LEN);
+        strncpy (diptr->fsType, sp->f_fstypename, DI_TYPE_LEN);
+
+        if (di_lib_debug > 1)
+        {
+            printf ("%s: %s\n", diptr->name, diptr->fsType);
+            printf ("\tbsize:%ld  frsize:%ld\n", (long) sp->f_bsize,
+                    (long) sp->f_frsize);
+#if _siz_long_long >= 8
+            printf ("\tblocks: tot:%llu free:%lld avail:%llu\n",
+                   sp->f_blocks, sp->f_bfree, sp->f_bavail);
+            printf ("\tinodes: tot:%llu free:%llu avail:%llu\n",
+                    sp->f_files, sp->f_ffree, sp->f_favail);
+#else
+            printf ("\tblocks: tot:%lu free:%lu avail:%lu\n",
+                   sp->f_blocks, sp->f_bfree, sp->f_bavail);
+            printf ("\tinodes: tot:%lu free:%lu avail:%lu\n",
+                    sp->f_files, sp->f_ffree, sp->f_favail);
+#endif
+        }
+    }
+
+    free ((char *) mntbufp);
+    return 0;
+}
+
+#endif /* _lib_getvfsstat */
+
 #if _lib_getmnt
 
 /*
@@ -1583,7 +1705,8 @@ di_getDiskEntries (diskInfo, diCount)
         /* convertMountOptions (vmtp->vmt_flags, diptr); */
         trimChar (diptr->options, ',');
 
-        strncpy (diptr->mountTime, ctime (&vmtp->vmt_time), DI_MNT_TIME_LEN);
+        strncpy (diptr->mountTime, ctime ((time_t *) &vmtp->vmt_time),
+                DI_MNT_TIME_LEN);
         bufp += vmtp->vmt_length;
 
         if (di_lib_debug > 0)
@@ -1668,7 +1791,9 @@ di_getDiskInfo (diskInfo, diCount)
 
 #endif /* _statfs_2arg */
 
-#if _lib_GetDiskFreeSpace
+#if _lib_GetDiskFreeSpace && \
+    _lib_GetDriveType && \
+    _lib_GetLogicalDriveStrings
 
 /*
  * di_getDiskInfo
@@ -1697,13 +1822,13 @@ di_getDiskEntries (diskInfo, diCount)
     char            buff [MSDOS_BUFFER_SIZE];
 
 
-#if _lib_GetDiskFreeSpaceEx
+# if _lib_GetDiskFreeSpaceEx
     if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: GetDiskFreeSpaceEx\n"); }
-#else
-# if _lib_GetDiskFreeSpace
+# else
+#  if _lib_GetDiskFreeSpace
     if (di_lib_debug > 0) { printf ("# lib:getDiskEntries: GetDiskFreeSpace\n"); }
+#  endif
 # endif
-#endif
     diskflag = DI_PRNT_IGNORE;
     rc = GetLogicalDriveStrings (MSDOS_BUFFER_SIZE, buff);
     *diCount = rc / BYTES_PER_LOGICAL_DRIVE;
@@ -1748,6 +1873,10 @@ di_getDiskEntries (diskInfo, diCount)
     return *diCount;
 }
 
+#endif  /* _lib_GetDiskFreeSpace || _lib_GetDiskFreeSpaceEx */
+
+#if _lib_GetVolumeInformation
+
 /*
  * di_getDiskInfo
  *
@@ -1877,7 +2006,7 @@ di_getDiskInfo (diskInfo, diCount)
     } /* for each mounted drive */
 }
 
-#endif /* _lib_GetDiskFreeSpace || _lib_GetDiskFreeSpaceEx */
+#endif  /* _lib_GetVolumeInformation */
 
 
 /*
@@ -1929,10 +2058,66 @@ convertMountOptions (flags, diptr)
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
-#if defined (MNT_SYNCHRONOUS)
-    if ((flags & MNT_SYNCHRONOUS) == MNT_SYNCHRONOUS)
+#if defined (MNT_FORCE)
+    if ((flags & MNT_FORCE) == MNT_FORCE)
     {
-        strncat (diptr->options, "sync,",
+        strncat (diptr->options, "force,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_GRPID)
+    if ((flags & MNT_GRPID) == MNT_GRPID)
+    {
+        strncat (diptr->options, "grpid,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_MAGICLINKS)
+    if ((flags & MNT_MAGICLINKS) == MNT_MAGICLINKS)
+    {
+        strncat (diptr->options, "magiclinks,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_MLSD)
+    if ((flags & MNT_MLSD) == MNT_MLSD)
+    {
+        strncat (diptr->options, "mlsd,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_NOATIMES)
+    if ((flags & MNT_NOATIMES) == MNT_NOATIMES)
+    {
+        strncat (diptr->options, "noatime,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_NOCACHE)
+    if ((flags & MNT_NOCACHE) == MNT_NOCACHE)
+    {
+        strncat (diptr->options, "nocache,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_NOCOREDUMP)
+    if ((flags & MNT_NOCOREDUMP) == MNT_NOCOREDUMP)
+    {
+        strncat (diptr->options, "nocoredump,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_NODEV)
+    if ((flags & MNT_NODEV) == MNT_NODEV)
+    {
+        strncat (diptr->options, "nodev,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_NODEVMTIME)
+    if ((flags & MNT_NODEVMTIME) == MNT_NODEVMTIME)
+    {
+        strncat (diptr->options, "nodevmtime,",
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
@@ -1950,31 +2135,10 @@ convertMountOptions (flags, diptr)
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
-#if defined (MNT_NODEV)
-    if ((flags & MNT_NODEV) == MNT_NODEV)
+#if defined (MNT_QUOTA)
+    if ((flags & MNT_QUOTA) == MNT_QUOTA)
     {
-        strncat (diptr->options, "nodev,",
-                DI_OPT_LEN - strlen (diptr->options) - 1);
-    }
-#endif
-#if defined (MNT_NOATIMES)
-    if ((flags & MNT_NOATIMES) == MNT_NOATIMES)
-    {
-        strncat (diptr->options, "noatime,",
-                DI_OPT_LEN - strlen (diptr->options) - 1);
-    }
-#endif
-#if defined (MNT_GRPID)
-    if ((flags & MNT_GRPID) == MNT_GRPID)
-    {
-        strncat (diptr->options, "grpid,",
-                DI_OPT_LEN - strlen (diptr->options) - 1);
-    }
-#endif
-#if defined (MNT_MLSD)
-    if ((flags & MNT_MLSD) == MNT_MLSD)
-    {
-        strncat (diptr->options, "mlsd,",
+        strncat (diptr->options, "quota,",
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
@@ -1992,24 +2156,17 @@ convertMountOptions (flags, diptr)
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
-#if defined (MNT_THROTTLE)
-    if ((flags & MNT_THROTTLE) == MNT_THROTTLE)
+#if defined (MNT_SOFTDEP)
+    if ((flags & MNT_SOFTDEP) == MNT_SOFTDEP)
     {
-        strncat (diptr->options, "secure,",
+        strncat (diptr->options, "softdep,",
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
-#if defined (MNT_QUOTA)
-    if ((flags & MNT_QUOTA) == MNT_QUOTA)
+#if defined (MNT_SYMPERM)
+    if ((flags & MNT_SYMPERM) == MNT_SYMPERM)
     {
-        strncat (diptr->options, "quota,",
-                DI_OPT_LEN - strlen (diptr->options) - 1);
-    }
-#endif
-#if defined (MNT_FORCE)
-    if ((flags & MNT_FORCE) == MNT_FORCE)
-    {
-        strncat (diptr->options, "force,",
+        strncat (diptr->options, "symperm,",
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
@@ -2020,10 +2177,31 @@ convertMountOptions (flags, diptr)
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
-#if defined (MNT_NOCACHE)
-    if ((flags & MNT_NOCACHE) == MNT_NOCACHE)
+#if defined (MNT_SYNCHRONOUS)
+    if ((flags & MNT_SYNCHRONOUS) == MNT_SYNCHRONOUS)
     {
-        strncat (diptr->options, "nocache,",
+        strncat (diptr->options, "sync,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_THROTTLE)
+    if ((flags & MNT_THROTTLE) == MNT_THROTTLE)
+    {
+        strncat (diptr->options, "throttle,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_UNION)
+    if ((flags & MNT_UNION) == MNT_UNION)
+    {
+        strncat (diptr->options, "union,",
+                DI_OPT_LEN - strlen (diptr->options) - 1);
+    }
+#endif
+#if defined (MNT_UNION)
+    if ((flags & MNT_UNION) == MNT_UNION)
+    {
+        strncat (diptr->options, "union,",
                 DI_OPT_LEN - strlen (diptr->options) - 1);
     }
 #endif
